var m = require('mithril');
var Visualizer = require('./js/modules/visualizer');
var Print = require('./js/print/print');
var Postprocessor = require('./js/postprocessor');

Main = {}

Main.controller = function() {
  this.post = m.prop(new Postprocessor());
  this.print = m.prop(Print.create());
}

Main.view = function(ctrl) {
  return [
    m('div#loadingView', [
      m('div#loadingMessage', [
        m('svg', {height: 400, width: 500}, [
          m('polygon', {points: '239,23 299,209 255,230 213,106', style: 'fill:#a700ff;stroke:#a700ff;stroke-width:1'}),
          m('polygon', {points: '212,108 155,276 255,230', style: 'fill:#fede00;stroke:#fede00;stroke-width:1'}),
          m('polygon', {points: '110,297 255,230 294,353 93,354', style: 'fill:#00eec9;stroke:#00eec9;stroke-width:1'}),
        ])
      ])
    ]),
    m("div#vizBox", { config: drawVisualizer }),
    m("div.topbar", [
      m("div.leftTools", [
        m("button#loadCode", { onclick: ctrl.post().openFile.bind(ctrl.print()) }, "Load G-Code"),

        m('div#divider'),
        ctrl.print().driveColour.map(function(color, index) {
          return [
            m('svg', { width: 20, height: 20 }, [
              m('rect', { x: 0, y: 0, width: 20, height: 20, rx: 5, ry: 5, style: 'fill:' + color }),
            ]),
            m('span#driveNumber', 'Drive ' + (index + 1)),
          ];
        }),

      ]),

      m("div.rightTools", [
        m("div.paletteSelect"),
        m("div.printerSelect"),
        m('div#divider'),
        m("button#processCode", { onclick: ctrl.post().saveFile.bind(ctrl.print()), disabled: true }, "Process for Printer"),
      ]),
    ]),

    // Empty divs that will be filled dynamically
    m("div.left-pane"),
    m("div.right-pane"),
    m("div#open-modal.modal-popup"),
  ]
}

/**
 * drawVisualizer() checks to see if a Visualizer model has been created on screen
 * and creates it, if not, otherwise lets it be. Prevents redrawing on every action.
 */
function drawVisualizer(element, isInit, context) {
   if (!isInit) new Visualizer();
}