var Calibrator = require('./calibrator');
var config = require('../../configuration');
var common = require('../common');

function showValidationError( element, message ) {
  m.render( document.querySelector("#errorText"), message );
  element.style.backgroundColor = '#FFCCCC';
  element.focus();
}

function resetValidationError( element ) {
  m.render( document.querySelector("#errorText"), "" );
  element.style.backgroundColor = 'white';
}

Calibrator.prototype.showPaletteWindow = function() {
  var _this = this;

  calibWindow = document.querySelector("#open-modal");

  m.render(calibWindow, [
    m("div.container", [
      m("div.save", [
        m("h2", "Palette Settings"),
        m("span#errorText"),
        m("br"),

        m("label#paletteName", "Palette Name"), m("input#paletteName", {
          onchange: m.withAttr("value", function (value) {
            _this.paletteName = value;
          })
        }),

        m("label#stepsPerCount", "Steps Per Count"), m("input#stepsPerCount", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#stepsPerCount");
            if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Steps Per Count cannot be 0." );
            } else if( isNaN( value ) ) {
              showValidationError( element, "Your Steps Per Count is not a number." );
            } else if( value < 2 || value > 4 ) {
              showValidationError( element, "Did you enter your Steps Per Count incorrectly?" );
            } else {
              _this.stepsPerCount = value;
              resetValidationError( element );
            }
          })
        }),

        m("label#spliceDist", "Splice Distance"), m("input#spliceDist", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#spliceDist");
            if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Splice Distance cannot be 0." );
            } else if( isNaN( value ) ) {
              showValidationError( element, "Your Splice Distance is not a number." );
            } else if( value < 90 || value > 440 ) {
              showValidationError( element, "Did you enter your Splice Distance incorrectly?" );
            } else {
              _this.spliceDist = value;
              resetValidationError( element );
            }
          })
        }),

        m("label#cutterOptoDist", "Cutter to Opto Distance"), m("input#cutterOptoDist", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#cutterOptoDist");
            if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Cutter to Opto Distance cannot be 0." );
            } else if( isNaN( value ) ) {
              showValidationError( element, "Your Cutter to Opto Distance is not a number." );
            } else if( value < 325 || value > 1700 ) {
              showValidationError( element, "Did you enter your Cutter to Opto Distance incorrectly?" );
            } else {
              _this.cutterOptoDist = value;
              resetValidationError( element );
            }
          })
        }),

        m("button", { onclick: _this.savePaletteSettings.bind(_this) }, "Save Settings"),
      ]),

      m("div.edit", [
        m("h2", "Saved Settings"),

        m("select[multiple]", {
          onchange: function(selected) {
            _this.selectedPalette = common.populatePaletteList()[selected.srcElement.selectedIndex];
          }
        }, [
          common.populatePaletteList().map(function (palette, index) {
            return m("option", palette[0].paletteName);
          })
        ]),

        m("button", { onclick: function() {
          _this.deleteSetting("palette", _this.selectedPalette[1]);
        } }, "Delete"),
      ]),

    ]),
  ]);

  window.onkeydown = function(evt) {
    evt = evt || window.event;
    if (evt.keyCode == 27) {
      _this.closeOpenWindow();
    }
  };

  calibWindow.style.opacity = 1;
  calibWindow.style.pointerEvents = "auto";
};

/** Totally not ready for production, but will be used to house Palette-specific settings
that firmware will be using. */
Calibrator.prototype.savePaletteSettings = function() {
  var _this = this;
  var settingsObj = config.load();

  var calibWindow = document.querySelector("#open-modal");
  calibWindow.style.opacity = 0;
  calibWindow.style.pointerEvents = "none";

  var currentDate = Date.now();

  if (settingsObj.app.palette == undefined) {
    settingsObj.app.palette = {};
  }

  settingsObj.app.palette[currentDate] = {
    "paletteName": _this.paletteName,
    "stepsPerCount": _this.stepsPerCount,
    "spliceDist": _this.spliceDist,
    "cutterOptoDist": _this.cutterOptoDist,
  };

  config.dump(settingsObj);
  common.populateMachineList();
  alert('Successfully saved!');
};