var Calibrator = require('./calibrator');
var config = require('../../configuration');
var common = require('../common');

function showValidationError( element, message ) {
  m.render( document.querySelector("#errorText"), message );
  element.style.backgroundColor = '#FFCCCC';
  element.focus();
}

function resetValidationError( element ) {
  m.render( document.querySelector("#errorText"), "" );
  element.style.backgroundColor = 'white';
}

Calibrator.prototype.showPrinterWindow = function() {
  var _this = this;
  _this.originMiddle = false;
  console.log(_this);

  calibWindow = document.querySelector("#open-modal");

  m.render(calibWindow, [
    m("div.container", [
      m("div.save", [
        m("h2", "Printer Settings"),
        m("span#errorText"),
        m("br"),

        m("label#printerName", "Printer Name"), m("input#printerName", {
          onchange: m.withAttr("value", function (value) {
            _this.printerName = value;
          })
        }),

        m("label#loadingOffset", "Loading Offset"), m("input#loadingOffset", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#loadingOffset");
            if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Printer Value cannot be 0." );
            } else if ( isNaN( value ) ) {
              showValidationError( element, "Your Printer Value is not a number." );
            } else {
              _this.loadingOffset = value;
              resetValidationError( element );
            }
          })
        }),

        m("label#printerValue", "Print Value"), m("input#printerValue", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#printerValue");
            if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Print Value cannot be 0." );
            } else if ( isNaN( value ) ) {
              showValidationError( element, "Your Print Value is not a number." );
            } else {
              _this.printerValue = value;
              resetValidationError( element );
            }
          })
        }),

        m("button#loadCode", { onclick: _this.openFile.bind(_this), style: 'margin-right:10px;' }, "LOAD CALIBRATION G-CODE"),
        m("span#printLength", "0.0 mm" ),
        m("br"),

        m("br"),

        m("label#printBedDimensions", "Print Bed Size (mm)"),
        m("div#printBedDimensions", [
          m("input#printBedLength[placeholder=Length]", {
            onchange: m.withAttr("value", function (value) {
              var element = document.querySelector("input#printBedLength");
              if ( parseFloat(value) == 0 ) {
                showValidationError( element, "Your Printer Bed Length cannot be 0." );
              } else if ( isNaN( value ) ) {
                showValidationError( element, "Your Printer Bed Length is not a number." );
              } else {
                _this.printBedLength = value;
                resetValidationError( element );
              }
            })
          }),
          m("input#printBedWidth[placeholder=Width]", {
            onchange: m.withAttr("value", function (value) {
              var element = document.querySelector("input#printBedWidth");
              if ( parseFloat(value) == 0 ) {
                showValidationError( element, "Your Printer Bed Width cannot be 0." );
              } else if ( isNaN( value ) ) {
                showValidationError( element, "Your Printer Bed Width is not a number." );
              } else {
                _this.printBedWidth = value;
                resetValidationError( element );
              }
            })
          }),
        ]),

        m("label#transitionTowerVolume", "Transition Tower Volume"), m("input#transitionTowerVolume", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#transitionTowerVolume");
            if ( parseFloat(value) > 100 ) {
              showValidationError( element, "Your Tower Volume is too large." );
            } else if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Tower Volume cannot be 0." );
            } else if ( isNaN( value ) ) {
              showValidationError( element, "Your Tower Volume is not a number." );
            } else {
              _this.transitionTowerVolume = value;
              resetValidationError( element );
            }
          })
        }),

        m("label#transitionTowerFastRate", "Transition Tower Fast Rate"), m("input#transitionTowerFastRate", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#transitionTowerFastRate");
            if ( parseFloat(value) > 12000 ) {
              showValidationError( element, "Your Tower Fast Rate is too fast!" );
            } else if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Tower Fast Rate cannot be 0." );
            } else if ( isNaN( value ) ) {
              showValidationError( element, "Your Tower Fast Rate is not a number!" );
            } else {
              _this.transitionTowerFastRate = value;
              resetValidationError( element );
            }
          })
        }),

        m("label#transitionTowerSlowRate", "Transition Tower Slow Rate"), m("input#transitionTowerSlowRate", {
          onchange: m.withAttr("value", function (value) {
            var element = document.querySelector("input#transitionTowerSlowRate");
            if ( parseFloat(value) > 10000 ) {
              showValidationError( element, "Your Tower Slow Rate is too fast!" );
            } else if ( parseFloat(value) == 0 ) {
              showValidationError( element, "Your Tower Slow Rate cannot be 0." );
            } else if ( isNaN( value ) ) {
              showValidationError( element, "Your Tower Slow Rate is not a number!" );
            } else {
              _this.transitionTowerSlowRate = value;
              resetValidationError( element );
            }
          })
        }),

        m("br"),

        m("label#bedOrigin", "Bed Origin Middle?"),
        m("input[type=checkbox]", { onclick: m.withAttr("checked", function(value) {
          _this.originMiddle = value;
        }), checked: _this.originMiddle } ),

        m("button", { onclick: function() {
          if ( validatePrinterSettingsOnSubmit(_this) ) {
            if ( (( _this.printerValue / _this.Etotal ) * 1.01) > 40 ) {
              m.render( document.querySelector("#errorText"), "Please double check your printer value, it seems high!" );
            } else if ( (( _this.printerValue / _this.Etotal ) * 1.01) < 15 ) {
              m.render( document.querySelector("#errorText"), "Please double check your printer value, it seems too low!" );
            } else {
              _this.savePrinterSettings();
            };
          } else {
            m.render( document.querySelector("#errorText"), "Please fill out all fields!" );
          }
        } }, "Save Settings"),
      ]),

      m("div.edit", [
        m("h2", "Saved Settings"),

        m("select[multiple]", {
          onchange: function(selected) {
            _this.selectedMachine = common.populateMachineList()[selected.srcElement.selectedIndex];
          }
        }, [
          common.populateMachineList().map(function (machine, index) {
            return m("option", machine[0].name);
          })
        ]),

        m("button", { onclick: function() {
          _this.deleteSetting("printer", _this.selectedMachine[1]);
        } }, "Delete"),
      ]),
    ]),
  ]);

  window.onkeydown = function(evt) {
    evt = evt || window.event;
    if (evt.keyCode == 27) {
      _this.closeOpenWindow();
    }
  };

  calibWindow.style.opacity = 1;
  calibWindow.style.pointerEvents = "auto";
};

Calibrator.prototype.savePrinterSettings = function() {
  var _this = this;
  var settingsObj = config.load();

  var calibWindow = document.querySelector("#open-modal");
  calibWindow.style.opacity = 0;
  calibWindow.style.pointerEvents = "none";

  var currentDate = Date.now();

  if (settingsObj.app.printer == undefined) {
    settingsObj.app.printer = {};
  }

  settingsObj.app.printer[currentDate] = {
    "name": _this.printerName,
    "printerValue": _this.printerValue,
    "pulsesPerMM": ( _this.printerValue / _this.Etotal ) * 1.01, // Added by Derek, Mar 18, 2016
    "loadingOffset": _this.loadingOffset,
    "printBedX": _this.printBedLength,
    "printBedY": _this.printBedWidth,
    "printBedOriginMiddle": _this.originMiddle,
    "towerVolume": _this.transitionTowerVolume,
    "towerFastRate": _this.transitionTowerFastRate,
    "towerSlowRate": _this.transitionTowerSlowRate,
  };

  config.dump(settingsObj);
  common.populateMachineList();
  alert('Successfully saved!');
};

function validatePrinterSettingsOnSubmit( calibModel ) {
  var _this = calibModel;

  // Created from expected properties on save
  var settingsArray = [
    "printerName",
    "printerValue",
    "Etotal",
    "loadingOffset",
    "printBedLength",
    "printBedWidth",
    "originMiddle",
    "transitionTowerVolume",
    "transitionTowerFastRate",
    "transitionTowerSlowRate",
  ];

  for (var setting = 0; setting < settingsArray.length; setting++) {
    if (_this.hasOwnProperty( settingsArray[setting] ) === false) { return false; break; }
  }

  return true;
}