module.exports = Calibrator = (function() {
  var config = require('../../configuration');
  var common = require('../common');
  var Print = require('../print/print');

  Calibrator.create = function() {
    return new Calibrator();
  };

  function Calibrator() {
    require('./calibrator-palette');
    require('./calibrator-printer');
  };

  function showValidationError( element, message ) {
    m.render( document.querySelector("#errorText"), message );
    element.style.backgroundColor = '#FFCCCC';
    element.focus();
  }

  function resetValidationError( element ) {
    m.render( document.querySelector("#errorText"), "" );
    element.style.backgroundColor = 'white';
  }

  Calibrator.prototype.closeOpenWindow = function() {
    calibWindow = document.querySelector("#open-modal");
    m.render( document.querySelector("#errorText"), "" );

    calibWindow.style.opacity = 0;
    calibWindow.style.pointerEvents = "none";
  };

  Calibrator.prototype.openFile = function() {
    var _this = this;
    var printModel = Print.create();

    dialog.showOpenDialog({ filters: [ { name: 'G-code', extensions: ['gcode'] } ] }, function (fileNames) {
      if (fileNames === undefined) return;
      var fileName = fileNames[0];
      fs.readFile(fileName, 'utf-8', function (err, data) {

        if (err) {
          console.log("Oh no!");
        } else {

          printModel.inputFile = fileName;
          printModel.preFlight(function() {
            printModel.parseGcode(function() {
              _this.Etotal = printModel.Etotal;
              m.render(document.querySelector("#printLength"), _this.Etotal + " mm");
            });
          });

        }
      });
    });
  };

  Calibrator.prototype.deleteSetting = function(profileType, index) {
    var settingsObj = config.load();

    console.log(profileType + ', ' + index);

    if (settingsObj.app != undefined) {
      if (settingsObj.app.hasOwnProperty(profileType)) {
        if (settingsObj.app[profileType].hasOwnProperty(index)) {
          console.log( "Deleting" + settingsObj.app[profileType][index] );
          delete settingsObj.app[profileType][index];
          config.dump(settingsObj);
          common.populateMachineList();
          // Ternary operator to refresh _correct_ window after deleting setting
          profileType === 'printer' ? this.showPrinterWindow() : this.showPaletteWindow();
          alert('Profile successfully deleted!');
        }
      }
    }
  }

  return Calibrator;

})();
