var exports = module.exports = {}

var config = require('../configuration');

exports.showLoadingView = function() {
  loadingView = document.querySelector("#loadingView");

  loadingView.style.opacity = 1;
  loadingView.style.pointerEvents = "auto";

  polygonList = loadingView.querySelector("svg").querySelectorAll("polygon");
  animate(polygonList);
}

var polygon, index;

function animate(polygonList) {
  if ( index == undefined ) { index = 0; }
  if ( index == polygonList.length ) { index = 0; }
  var polygon = polygonList[ index ];
  if ( index < polygonList.length ) {
    polygon.style.transition = "opacity 200ms ease-in";
    polygon.style.opacity = 0.5;
    setTimeout(function() {
      polygon.style.opacity = 1;
      index++;
      animate( polygonList );
    }, 350);
  }
}

exports.hideLoadingView = function() {
  loadingView = document.querySelector("#loadingView");

  loadingView.style.opacity = 0;
  loadingView.style.pointerEvents = "none";
}

exports.parseLine = function(line, info) {
  line = line.replace(/;.*$/, '').trim();
  if (line) {
    var params = line.split(' ');
    if (params) {
      if ( line.match(/(T0|T1|T2|T3)/) && !line.match( /(M6|M104|M109|M126|M127|M133|M134|M140)/ ) ) {
        var cmd = 'TOOLCHANGE';
      } else {
        var cmd = params[0];
      }
      var args = {
        'cmd': cmd
      };

      if (params.length == 1) {
        params.forEach( function(param) {
          var key = param[0].toLowerCase();
          var value = parseFloat(param.substring(1));
          args[key] = value;
        });
      } else if (params.length == 0) {
        // We doin' nothin, fam
      } else {
        params.splice(1).forEach( function(param) {
          if ( param != '' ) {
            var key = param[0].toLowerCase();
            var value = parseFloat(param.substring(1));
            args[key] = value;
          }
        });
      }
      return args;
    }
  }
};

exports.populateMachineList = function() {
  var configData = config.load();
  var machineConfigObj = new Array();

  if ( Object.keys(configData.app.printer).length > 0 ) {
    for(var index in configData.app.printer) {
      if (configData.app.printer.hasOwnProperty(index)) {
        machineConfigObj.push([configData.app.printer[index], index]);
      }
    }
    // setTimeout() used to push this enable code to after the event loop
    if ( document.querySelector("#loadCode") !== null ) {
      setTimeout( function() { document.querySelector("#loadCode").disabled = false; }, 0);
    }
  } else {
    machineConfigObj.push([{name: 'No machines! Please calibrate first.'}, 0]);
    if ( document.querySelector("#loadCode") !== null ) {
      setTimeout( function() { document.querySelector("#loadCode").disabled = true; }, 0);
    }
  }

  return machineConfigObj;
};

exports.populatePaletteList = function() {
  var configData = config.load();
  var paletteConfigObj = new Array();

  if ( Object.keys(configData.app.palette).length > 0 ) {
    for(var index in configData.app.palette) {
      if (configData.app.palette.hasOwnProperty(index)) {
        paletteConfigObj.push([configData.app.palette[index], index]);
      }
    }
  } else {
    paletteConfigObj.push([{paletteName: 'No Palettes! Please calibrate first.'}, 0]);
  }

  return paletteConfigObj;
};