module.exports = Firmware = (function() {

  var Avrgirl = require('avrgirl-arduino');
  var AvrgirlExtension = require('./modules/avrgirl-extension');

  // Essential: Postprocessor class is loaded when the user navigates to the
  // Post-Processor tab in navigation, and is where they can load their multi-colour
  // gcode file, separated by tool changes, for post-processing (removal of
  // every tool other than T0) and for creation of the SEEM file, for the Palette.

  Firmware.create = function() {
    return new Firmware();
  };

  function Firmware() { };

  Firmware.prototype.showFirmwareWindow = function () {
    var _this = this;

    firmwareWindow = document.querySelector("#open-modal");

    m.render(firmwareWindow, [
      m("div.container", [
        m("div.save", [
          m("h4", "Upload HEX File"),
          m("button#loadCode", { onclick: _this.openFile.bind(_this) }, "Load"),
          m("span#errorText", "" ),
        ]),
      ]),
    ]);

    window.onkeydown = function(evt) {
      evt = evt || window.event;
      if (evt.keyCode == 27) {
        _this.closeOpenWindow();
      }
    };

    firmwareWindow.style.opacity = 1;
    firmwareWindow.style.pointerEvents = "auto";

  };

  Firmware.prototype.closeOpenWindow = function() {
    firmwareWindow = document.querySelector("#open-modal");

    firmwareWindow.style.opacity = 0;
    firmwareWindow.style.pointerEvents = "none";
  };

  // Essential: prompts the user to select a gcode file for post-processing,
  // on success, prompts user to name the new file that will be created and the
  // location to save those files.
  //
  // Notes: re-work to not automatically prompt the save file dialog, should get
  // user to check off a few options before post-processing the file
  Firmware.prototype.openFile = function (controller) {
    var avrgirl = new Avrgirl({
      board: 'mega'
    });

    dialog.showOpenDialog({ filters: [ { name: 'Hex File', extensions: ['hex'] } ] }, function (fileNames) {
      if (fileNames === undefined) return;
      var fileName = fileNames[0];
      fs.readFile(fileName, 'utf-8', function (err, data) {
        if (err) {
          console.log("Oh no!");
        } else {
          avrgirl.flash(fileName, function (error) {
            if (error) {
              m.render(document.querySelector("#errorText"), error);
            } else {
              console.info('done.');
              m.render(document.querySelector("#errorText"), 'Successfully flashed Palette firmware!');
            }
          });
        }
      });
    });
  }

  return Firmware;

})();