module.exports = Tower = (function() {
  Tower.create = function(printModel) {
    return new Tower(printModel);
  };

  function Tower(printModel) {
    this.init(printModel);
  }

  Tower.prototype.init = function(printModel) {
    this.printModel = printModel;
  	this.printBedDimensions = { length: 0, width: 0, height: 0};
  	this.modelBoundingBox = { maxX: 0, minX: 0, maxY: 0, minY: 0, maxZ: 0 };
  };

  Tower.prototype.transitionLayer = function(towerArray, e, toolChangeNum) {
    var _this = this;
    var towerStringArray = [];
    var x = towerArray.x;
    var y = towerArray.y;

    if (e == undefined) { e = -1.000; }

    // Used to calculate how much extrusion needs to be generated for a given distance
    // to be covered
    var extrusionFactor = 30.06534219;
    if (towerArray.densityPerLayer != undefined && towerArray.densityPerLayer == 1) {
      extrudePercentageFactor = 0.03099814079;
    } else if (towerArray.densityPerLayer != undefined && towerArray.densityPerLayer == 0.1) {
      extrudePercentageFactor = 0.03849814079;
    }

    // Below values are all derived from studying existing g-code and the amount
    // of extrusion relative to the distance traveled for a specific command
    //
    // 0.03849814079 == single tower layer wall == 3.8% of the total layer volume
    var totalExtrusion = _this.printModel.towerVolume / 2.4242424242;
    var boundaryCalculation = _this.printModel.towerVolume / 3.1515151515;
    var distRequired = ( extrusionFactor * ( 0.03849814079 * totalExtrusion ) ).toFixed(3);
    var towerVertices = [
      [ parseFloat(x), parseFloat(y) ],
      [ parseFloat(x), parseFloat(y)+parseFloat(distRequired) ],
      [ parseFloat(x)-parseFloat(distRequired), parseFloat(y)+parseFloat(distRequired) ],
      [ parseFloat(x)-parseFloat(distRequired), parseFloat(y) ]
    ];

    var eValBuffer = parseFloat(0);

    if (towerArray.firstLayer == false) {
      var fastRate = _this.printModel.towerFastRate;
      var slowRate = _this.printModel.towerSlowRate;
    } else {
      var fastRate = (_this.printModel.towerFastRate / 2);
      var slowRate = (_this.printModel.towerSlowRate / 2);
    }

    var extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    if (_this.printModel.relativeExtrude === true) { } else { towerStringArray.push('G92 E0\n'); }

    towerStringArray.push('G0 F' + fastRate + ' X' + towerVertices[0][0] + ' Y' + towerVertices[0][1] + '\n');
    towerStringArray.push(';TYPE:SUPPORT\n');
    //towerStringArray.push('G1 F' + slowRate + ' E' + 1.0000 + '\n');

    if ( towerArray.isPinging == true ) {
      towerStringArray.push('G4 P6000\n');
      //towerStringArray.push('G1\n');
      //towerStringArray.push('G4 P4000\n');
    }

    towerStringArray.push('G1 F' + slowRate + ' E' + 1.0000 + '\n');

    if (_this.printModel.relativeExtrude === true) { } else { towerStringArray.push('G92 E0\n'); }

    // Drawing boundaries
    //if (_this.printModel.relativeExtrude === true) { } else { towerStringArray.push('G92 E0\n'); }
    towerStringArray.push('G1 F' + slowRate + ' X' + towerVertices[1][0] + ' Y' + towerVertices[1][1] + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + towerVertices[2][0] + ' Y' + towerVertices[2][1] + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + towerVertices[3][0] + ' Y' + towerVertices[3][1] + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + towerVertices[0][0] + ' Y' + towerVertices[0][1] + ' E' + extrudeAmount.toFixed(5) + '\n');

    // Second Wall

    towerStringArray.push('G0 X' + (towerVertices[0][0]+0.4) + ' Y' + (towerVertices[0][1]-0.4) + '\n');
    towerStringArray.push('G1 F' + slowRate + ' X' + (towerVertices[1][0]+0.4) + ' Y' + (towerVertices[1][1]+0.4) + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + (towerVertices[2][0]-0.4) + ' Y' + (towerVertices[2][1]+0.4) + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + (towerVertices[3][0]-0.4) + ' Y' + (towerVertices[3][1]-0.4) + ' E' + extrudeAmount.toFixed(5) + '\n');

    extrudeAmount = (0.03849814079*boundaryCalculation);
    eValBuffer += extrudeAmount;
    if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }

    towerStringArray.push('G1 X' + (towerVertices[0][0]+0.4) + ' Y' + (towerVertices[0][1]-0.4) + ' E' + extrudeAmount.toFixed(5) + '\n');

    if ( towerArray.isPinging == true ) {
      towerStringArray.push('G4 P6000\n');
      //towerStringArray.push('G1\n');
      //towerStringArray.push('G4 P4000\n');
    }

    // Rastering begins here

    var rasterDist = 0;
    if (towerArray.densityPerLayer != undefined) { var densityPerLayer = towerArray.densityPerLayer; }
      else { var densityPerLayer = 1; }
    var distSegment = Math.ceil(distRequired/( 0.4*(1/densityPerLayer) )); // Currently 0.4mm increments at 100% density
    var incrementSize = distRequired/distSegment;

    if ( towerArray.flippedDraw == true ) {
      var nextXY = { x: parseFloat(x), y: parseFloat(y)+parseFloat(distRequired) };
      var origXY = { x: parseFloat(x), y: parseFloat(y)+parseFloat(distRequired) };
      var untilXY = { x: parseFloat(x)-parseFloat(distRequired)-(incrementSize*densityPerLayer), y: parseFloat(y) };
    } else {
      var nextXY = { x: parseFloat(x), y: parseFloat(y) };
      var origXY = { x: parseFloat(x), y: parseFloat(y) };
      var untilXY = { x: parseFloat(x)-parseFloat(distRequired)-(incrementSize*densityPerLayer), y: parseFloat(y)+parseFloat(distRequired)-(incrementSize*densityPerLayer) };
    }

    function operate1(posHash) {
      if (towerArray.flippedDraw == true) {
        posHash.x -= incrementSize;
        posHash.y -= incrementSize;
      } else {
        posHash.x -= incrementSize;
        posHash.y += incrementSize;
      }
    }

    if (towerArray.flippedDraw == true) {
      var whichLine = 0;
      while( nextXY.x > untilXY.x && nextXY.y > untilXY.y ) {
        operate1(nextXY);
        rasterDist = Math.sqrt(Math.abs( Math.pow(parseFloat(origXY.x) - parseFloat(nextXY.x), 2) + Math.pow(parseFloat(origXY.y) - parseFloat(nextXY.y), 2) ) );
        extrudeAmount = ( ( rasterDist * extrudePercentageFactor ) / distRequired ) * boundaryCalculation;
        eValBuffer += extrudeAmount;
        if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }
        if (whichLine == 0) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(nextXY.x).toFixed(3) + ' Y' + parseFloat(origXY.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(origXY.x).toFixed(3) + ' Y' + parseFloat(nextXY.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 1;
        } else if (whichLine == 1) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(origXY.x).toFixed(3) + ' Y' + parseFloat(nextXY.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(nextXY.x).toFixed(3) + ' Y' + parseFloat(origXY.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 0;
        }
      };
    } else {
      var whichLine = 0;
      while( nextXY.x > untilXY.x && nextXY.y < untilXY.y ) {
        operate1(nextXY);
        rasterDist = Math.sqrt(Math.abs( Math.pow(parseFloat(origXY.x) - parseFloat(nextXY.x), 2) + Math.pow(parseFloat(origXY.y) - parseFloat(nextXY.y), 2) ) );
        extrudeAmount = ( ( rasterDist * extrudePercentageFactor ) / distRequired ) * boundaryCalculation;
        eValBuffer += extrudeAmount;
        if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }
        if (whichLine == 0) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(nextXY.x).toFixed(3) + ' Y' + parseFloat(origXY.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(origXY.x).toFixed(3) + ' Y' + parseFloat(nextXY.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 1;
        } else if (whichLine == 1) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(origXY.x).toFixed(3) + ' Y' + parseFloat(nextXY.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(nextXY.x).toFixed(3) + ' Y' + parseFloat(origXY.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 0;
        }
      };
    }

    if (towerArray.flippedDraw == true) {
      var nextXYZ = { x: parseFloat(x)-parseFloat(distRequired), y: parseFloat(y) };
      var origXYZ = { x: parseFloat(x)-parseFloat(distRequired), y: parseFloat(y) };
      var untilXYZ = { x: parseFloat(x)-(incrementSize*densityPerLayer), y: parseFloat(y)+parseFloat(distRequired) };
    } else {
      var nextXYZ = { x: parseFloat(x)-parseFloat(distRequired), y: parseFloat(y)+parseFloat(distRequired) };
      var origXYZ = { x: parseFloat(x)-parseFloat(distRequired), y: parseFloat(y)+parseFloat(distRequired) };
      var untilXYZ = { x: parseFloat(x)-(incrementSize*densityPerLayer), y: parseFloat(y)-(incrementSize*densityPerLayer) };
    }

    function operate2(posHash) {
      if (towerArray.flippedDraw == true) {
        posHash.x += incrementSize;
        posHash.y += incrementSize;
      } else {
        posHash.x += incrementSize;
        posHash.y -= incrementSize;
      }
    }

    if (towerArray.flippedDraw == true) {
      var whichLine = 0;
      while( nextXYZ.x < (untilXYZ.x-incrementSize) && nextXYZ.y < untilXYZ.y ) {
        operate2(nextXYZ);
        rasterDist = Math.sqrt(Math.abs( Math.pow(parseFloat(origXYZ.x) - parseFloat(nextXYZ.x), 2) + Math.pow(parseFloat(origXYZ.y) - parseFloat(nextXYZ.y), 2) ));
        extrudeAmount = ( ( rasterDist * extrudePercentageFactor ) / distRequired ) * boundaryCalculation;
        eValBuffer += extrudeAmount;
        if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }
        if (whichLine == 0) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(origXYZ.x).toFixed(3) + ' Y' + parseFloat(nextXYZ.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(nextXYZ.x).toFixed(3) + ' Y' + parseFloat(origXYZ.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 1;
        } else if (whichLine == 1) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(nextXYZ.x).toFixed(3) + ' Y' + parseFloat(origXYZ.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(origXYZ.x).toFixed(3) + ' Y' + parseFloat(nextXYZ.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 0;
        }
      };
    } else {
      var whichLine = 0;
      while( nextXYZ.x < (untilXYZ.x-incrementSize) && nextXYZ.y > (untilXYZ.y-incrementSize) ) {
        operate2(nextXYZ);
        rasterDist = Math.sqrt(Math.abs( Math.pow(parseFloat(origXYZ.x) - parseFloat(nextXYZ.x), 2) + Math.pow(parseFloat(origXYZ.y) - parseFloat(nextXYZ.y), 2) ));
        extrudeAmount = ( ( rasterDist * extrudePercentageFactor ) / distRequired ) * boundaryCalculation;
        eValBuffer += extrudeAmount;
        if (_this.printModel.relativeExtrude === true) { extrudeAmount; } else { extrudeAmount = eValBuffer; }
        if (whichLine == 0) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(origXYZ.x).toFixed(3) + ' Y' + parseFloat(nextXYZ.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(nextXYZ.x).toFixed(3) + ' Y' + parseFloat(origXYZ.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 1;
        } else if (whichLine == 1) {
          towerStringArray.push('G0 F' + fastRate + ' X' + parseFloat(nextXYZ.x).toFixed(3) + ' Y' + parseFloat(origXYZ.y).toFixed(3) + '\n');
          towerStringArray.push('G1 F' + slowRate + ' X' + parseFloat(origXYZ.x).toFixed(3) + ' Y' + parseFloat(nextXYZ.y).toFixed(3) + ' E' + extrudeAmount.toFixed(5) + '\n');
          whichLine = 0;
        }
      };
    }

    if (_this.printModel.relativeExtrude == true) {
      towerStringArray.push('G1 F' + slowRate + ' E' + (-6.000) + '\n');
    } else {
      towerStringArray.push('G1 F' + slowRate + ' E' + (eValBuffer-6) + '\n');
    }

    if (_this.printModel.relativeExtrude === true) { } else { towerStringArray.push( 'G92 E' + parseFloat(e) + '\n' ); }

    if ( towerArray.isPinging == true ) towerArray.isPinging = false;

    return towerStringArray;
  };

  return Tower;
})();