module.exports = Postprocessor = (function() {
  var config = require('../configuration');
  var common = require('./common');
  var fs = require('fs');
  var es = require('event-stream');

  function Postprocessor() {};

  Postprocessor.populatePrinterSelect = function(printModel) {
    var _this = printModel;

    printerSelect = document.querySelector(".printerSelect");
    printerSelect.style.transition = "400ms ease-in";

    m.render(printerSelect, [
      m("select", {
        onchange: function(selected) {
          if (selected.srcElement.selectedIndex == 0) {
            document.querySelector("#processCode").disabled = true;
          } else {
            document.querySelector("#processCode").disabled = false;
            var selection = common.populateMachineList()[selected.srcElement.selectedIndex-1];
            _this.headerArray.pulsesPerMM = parseFloat(selection[0].pulsesPerMM);
            _this.headerArray.loadingOffset = parseInt(selection[0].loadingOffset);
            _this.printBedDimensions.x = parseInt(selection[0].printBedX);
            _this.printBedDimensions.y = parseInt(selection[0].printBedY);
            _this.printBedDimensions.z = parseInt(selection[0].printBedZ);
            _this.printBedDimensions.originMiddle = selection[0].printBedOriginMiddle;
            _this.towerVolume = parseInt(selection[0].towerVolume);
            _this.towerFastRate = parseInt(selection[0].towerFastRate);
            _this.towerSlowRate = parseInt(selection[0].towerSlowRate);
            common.showLoadingView();

            printModel.towerToggle(function() {
              printModel.viz.renderObject( printModel );
              common.hideLoadingView();
            });
          }
        },
      }, [
        m("option", "Select a Printer Profile"),
        common.populateMachineList().map(function (machine) {
          return m("option", machine[0].name);
        })
      ]),

    ]);

    printerSelect.style.display = "inline";
  };

  Postprocessor.populatePaletteSelect = function(printModel) {
    var _this = printModel;

    document.querySelector(".printerSelect").querySelector("select").disabled = true;

    paletteSelect = document.querySelector(".paletteSelect");
    paletteSelect.style.transition = "400ms ease-in";

    m.render(paletteSelect, [
      m("select", {
        onchange: function(selected) {
          if (selected.srcElement.selectedIndex == 0) {
            document.querySelector(".printerSelect").querySelector("select").disabled = true;
          } else {
            document.querySelector(".printerSelect").querySelector("select").disabled = false;
            var selection = common.populatePaletteList()[selected.srcElement.selectedIndex-1];
            _this.headerArray.stepsPerCount = parseFloat(selection[0].stepsPerCount);
            _this.headerArray.spliceDist = parseInt(selection[0].spliceDist);
            _this.headerArray.cutterOptoDist = parseInt(selection[0].cutterOptoDist);
          }
        },
      }, [
        m("option", "Select a Palette Profile"),
        common.populatePaletteList().map(function (palette) {
          return m("option", palette[0].paletteName);
        })
      ]),

    ]);

    paletteSelect.style.display = "inline";
  };

  Postprocessor.prototype.openFile = function() {
    var printModel = this;

    dialog.showOpenDialog({ filters: [ { name: 'G-Code File', extensions: ['gcode'] } ] }, function (fileNames) {
      if (fileNames === undefined) return;
      var fileName = fileNames[0];
      fs.readFile(fileName, 'utf-8', function (err, data) {
        m.redraw.strategy('all');
        common.showLoadingView();

        if (err) {
          console.log("Oh no!");
        } else {
          printModel.init();

          printModel.inputFile = fileName;
          printModel.viz = Visualizer.create();

          printModel.preFlight(function() {
            printModel.parseGcode(function() {
              printModel.calculateFirstLayerBoundingBox(function() {
                printModel.visualizePrint(function() {
                  printModel.viz.renderObject( printModel );
                  Postprocessor.populatePrinterSelect( printModel );
                  Postprocessor.populatePaletteSelect( printModel );
                  common.hideLoadingView();
                });
              });
            });
          });

        }
      });
    });
  };

  Postprocessor.prototype.saveFile = function() {
    var printModel = this;

    dialog.showSaveDialog({
      title: 'Save MSF file as...',
      filters: [{ name: 'MSF', extensions: ['msf'] }]
    },
    function (fileName) {
      // Return as empty if no file name given
      if (fileName === undefined) return;

      printModel.outputFile = fileName;
      printModel.makeGcode(function() {
        printModel.makeSEEMfile(function() {
          //Visualizer.renderObject( printModel.vizModel.geometryArr );
        });
      });
    });
  };

  return Postprocessor;

})();
